﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using System.Web;
using System.Web.Security;
using EPiServer.Security;
using EPiServer.Shell.Search.Contracts;

namespace QuickChat
{
    [Export(typeof(ISearchProvider))]
    public class UsersSearchProvider : ISearchProvider
    {
        #region ISearchProvider Members

        /// <summary>
        /// Area that the provider mapps to, used for spotlight searching
        /// </summary>
        public string Area
        {
            get { return "CMS"; }
        }

        /// <summary>
        /// The category that the provider returns hits in
        /// </summary>
        public string Category
        {
            get { return "Users"; }
        }

        /// <summary>
        /// Executes a Search on the provider
        /// </summary>
        /// <param name="query">The query to execute</param>
        /// <returns>A list of search results</returns>
        public IEnumerable<SearchResult> Search(Query query)
        {
            List<SearchResult> results = new List<SearchResult>();

            //Get the wildcardsymbol for the providers
            string wildcardSymbol = "%";
            ProviderCapabilitySettings settings = null;
            if (Membership.Provider != null && ProviderCapabilities.Providers.TryGetValue(Membership.Provider.GetType(), out settings))
            {
                wildcardSymbol = settings.WildcardSymbol;
            }

            string searchQuery = query.SearchQuery + wildcardSymbol;

            //Search for users using userName
            int totalRecords = 0;
            MembershipUserCollection memershipHits = Membership.FindUsersByName(searchQuery, 0, 10, out totalRecords);

            foreach (MembershipUser user in memershipHits)
            {
                //Do not exceed maximum number of results
                if (results.Count < query.MaxResults)
                {
                    AddUser(user, results);
                }
            }

            if (results.Count < query.MaxResults)
            {
                //Search bo emailadress
                memershipHits = Membership.FindUsersByEmail(searchQuery, 0, 10, out totalRecords);
                foreach (MembershipUser user in memershipHits)
                {
                    //Do not exceed maximum number of results
                    if (results.Count < query.MaxResults)
                    {
                        AddUser(user, results);
                    }
                }
            }

            return results;
        }

        #endregion

        #region Private Helpers

        private static void AddUser(MembershipUser user, IList<SearchResult> results)
        {
            string url = EPiServer.UriSupport.ResolveUrlFromUIBySettings("Admin/EditUser.aspx?membershipUsername=" + user.UserName + "&Provider=" + user.ProviderName);

            bool exist = results.FirstOrDefault(r => r.Url == url) != null;

            if (!exist)
            {
                results.Add(new SearchResult(url, user.UserName));
            }
        }

        #endregion
    }
}
